/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "overworld.h"
#include "util/memory.h"
#include "assert/assert.h"

overworld_t OVERWORLD;

void overworldInit() {
  memoryZero(&OVERWORLD, sizeof(overworld_t));

  // Test
  OVERWORLD.mapWidth = 16;
  OVERWORLD.mapHeight = 16;
  OVERWORLD.mapLayerCount = 1;

  memset(&OVERWORLD.tileIds, 0x01, sizeof(OVERWORLD.tileIds));

  assertTrue(
    (
      OVERWORLD.mapWidth * OVERWORLD.mapHeight * OVERWORLD.mapLayerCount
    ) <= OVERWORLD_TILE_COUNT_MAX,
    "Map size exceeds tile count."
  );

  entityInit(OVERWORLD.entities + OVERWORLD.entityCount++, ENTITY_TYPE_PLAYER);
  entityInit(OVERWORLD.entities + OVERWORLD.entityCount++, ENTITY_TYPE_NPC);
  OVERWORLD.entities[1].x = 2;
  OVERWORLD.entities[1].y = 2;
}

void overworldSceneInit() {

}

void overworldSceneDeinit() {

}

void overworldUpdate() {
  uint8_t i = 0;
  while(i < OVERWORLD.entityCount) {
    entityUpdate(OVERWORLD.entities + i++);
  }
}

uint32_t overworldTileGetIndex(
  const uint8_t layer,
  const uint8_t x,
  const uint8_t y
) {
  return (
    (layer * OVERWORLD.mapWidth * OVERWORLD.mapHeight) +
    (y * OVERWORLD.mapWidth) +
    x
  );
}

tileid_t overworldTileGet(
  const uint8_t layer,
  const uint8_t x,
  const uint8_t y
) {
  return OVERWORLD.tileIds[overworldTileGetIndex(layer, x, y)];
}

entity_t * overworldEntityGetAtPosition(const uint8_t x, const uint8_t y) {
  uint8_t i = 0;
  while(i < OVERWORLD.entityCount) {
    entity_t * entity = OVERWORLD.entities + i++;
    if(entity->x == x && entity->y == y) return entity;
  }
  return NULL;
}