/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "overworlddefs.h"
#include "overworld/entity/entity.h"
#include "overworld/map/tile.h"

typedef struct {
  entity_t entities[OVERWORLD_ENTITY_COUNT_MAX];
  uint8_t entityCount;

  tileid_t tileIds[OVERWORLD_TILE_COUNT_MAX];
  tiledata_t tileData[OVERWORLD_TILE_COUNT_MAX];
  uint8_t mapWidth, mapHeight, mapLayerCount;
} overworld_t;

extern overworld_t OVERWORLD;

/**
 * Initializes the overworld.
 */
void overworldInit();

/**
 * Called by the scene system when the overworld scene is made to be the 
 * active scene.
 */
void overworldSceneInit();

/**
 * Called by the scene system when the overworld scene is made to be inactive.
 */
void overworldSceneDeinit();

/**
 * Called by the scene system when the overworld scene is to perform a tick.
 */
void overworldUpdate();

/**
 * Platform level render method. Refer to sceneRender for information.
 */
void overworldRender();

/**
 * Returns the index of the tile at the given position.
 * 
 * @param layer The layer to get the tile from.
 * @param x The x position of the tile.
 * @param y The y position of the tile.
 * @return The index of the tile at the given position.
 */
uint32_t overworldTileGetIndex(
  const uint8_t layer,
  const uint8_t x,
  const uint8_t y
);

/**
 * Returns the tile at the given position.
 * 
 * @param layer The layer to get the tile from.
 * @param x The x position of the tile.
 * @param y The y position of the tile.
 * @return The tile id at the given position.
 */
tileid_t overworldTileGet(
  const uint8_t layer,
  const uint8_t x,
  const uint8_t y
);

/**
 * Returns the entity at the given position.
 * 
 * @param x The x position of the entity.
 * @param y The y position of the entity.
 * @return The entity at the given position.
 */
entity_t * overworldEntityGetAtPosition(
  const uint8_t x,
  const uint8_t y
);